#include "menu.h"
#include "config.h"
#include "gfxinterface.h"
#include "highscore.h"
#include "input.h"
#include "pix.h"
#include "time_util.h"
#include "xpms.h"
#include <iomanip>
#include <sstream>


const char *const menu_lines[] = {
	"XGalaga++",
	"v0.5 (2004-01-19)",
	"",
	"2003, 2004 Marc Mongenet, graphics from XGalaga 2.0.34",
	"This game is free software, covered by the GPL.",
	"",
	"Key             Action",
	"             ",
	"s               start",
	"q               quit",
	"p               pause",
	"1,2,3,4         set window size",
	"+               increase details (slower)",
	"-               decrease details (faster)",
	"<space bar>     fire",
	"<left arrow>    move left",
	"<right arrow>   move right",
	"",
	"Bonus",
	"",
	"extra shield every 1000 points"
};


StartMenu * StartMenu::singleton_ (0);


StartMenu::StartMenu()
: font_(X11::Inst().LoadQueryFont("*-fixed-bold-r-*-*-14-*-*-*-*-*-iso8859-1"))
, text_width_  (0)
, wait_scores_ (5000)
, print_scores_(false)
, green_comp_  (0)
, green_add_   (1024)
{
	if (!font_) {
		font_ = X11::Inst().LoadQueryFont("*-fixed-*");
		if (!font_) throw std::runtime_error ("X font error");
	}

	for (size_t i (0); i < sizeof menu_lines / sizeof menu_lines[0]; ++i) {
		text_width_ = std::max(text_width_, XTextWidth(font_, menu_lines[i], std::strlen(menu_lines[i])));
	}
}


StartMenu::~StartMenu()
{
	X11::Inst().FreeFont(font_);
}


bool StartMenu::Display(std::vector<StarsField> * stars_fields, Score * score)
{
	Input input;
        for (;;) {
		const double start_frame_time (CurrentMicroSecond());
		input.Update();
		if (input.Quit()) return false;
		if (input.Start()) {
			print_scores_ = true;
			wait_scores_ = 5000;
			return true;
		}
		Details::Instance().Change(input.Details());
		SetWindowSize(input.WindowSize());
		X11::Inst().GetWindowAttributes();
		ScrollStarFields(stars_fields);
		score->Refresh();
		if (--wait_scores_ < 0) {
			print_scores_ = !print_scores_;
			wait_scores_ = 1000;
			X11::Inst().ClearWindow();
		}
		if (print_scores_) PrintScores();
		else PrintHelp();
		X11::Inst().Sync(False);
		WaitNextFrame(start_frame_time);
	}
}


XColor StartMenu::NextColor()
{
	XColor color;
	color.red = 48*1024;
	color.blue = 65535;
	green_comp_ += green_add_;
	if (green_comp_ == 0) {
		green_add_ = - green_add_;
		green_comp_ += green_add_;
	}
	color.green = green_comp_;
	return color;
}


void StartMenu::PrintHelp()
{
	Coord pos (std::max(0, (X11::Inst().WindowWidth() - text_width_) / 2), 20);
	XColor color (NextColor());
	X11::Inst().AllocColorAlways(&color);
	X11::Inst().SetForeground(color.pixel);
	X11::Inst().SetClipMask(None);
	X11::Inst().SetFont(font_->fid);
	for (size_t i (0); i < sizeof menu_lines / sizeof menu_lines[0]; ++i) {
		X11::Inst().DrawString(pos, menu_lines[i]);
		pos.y += LineHeight();
		if (!menu_lines[i][0]) {
			color.red -= 8192;
			X11::Inst().AllocColorAlways(&color);
			X11::Inst().SetForeground(color.pixel);
		}
	}

	static const char **const bonuses[] =
		{ bonus_speed_xpm, bonus_fire_xpm, bonus_shield_xpm };
	static const char *const bonus_text[] =
		{ "extra speed", "extra fire", "extra shield" };
	for (size_t i (0); i < sizeof bonuses / sizeof bonuses[0]; ++i) {
		const Pix *const bonus (PixKeeper::Instance().Get(bonuses[i]));
		bonus->Draw(pos + Coord(bonus->Width() / 2, -bonus->Height() / 2));
		X11::Inst().SetClipMask(None);
		X11::Inst().DrawString(pos + Coord(2 * bonus->Width(), 0), bonus_text[i]);
		pos.y += std::max(bonus->Height(), LineHeight());
	}
}


void StartMenu::PrintScores()
{
	std::ostringstream ost;
	const int window_width (X11::Inst().WindowWidth());
	const int window_height (X11::Inst().WindowHeight());
	Coord pos (std::max(0, (window_width - text_width_) / 2), 20);
	XColor color (NextColor());
	X11::Inst().AllocColorAlways(&color);
	X11::Inst().SetForeground(color.pixel);
	X11::Inst().SetClipMask(None);
	X11::Inst().SetFont(font_->fid);

	ost <<"High scores for "<<window_width<<''<<window_height<<" window";
	X11::Inst().DrawString(pos, ost.str().c_str());
	pos.y += LineHeight();
	ost.str(std::string(ost.str().size(), ''));
	X11::Inst().DrawString(pos, ost.str().c_str());
	pos.y += LineHeight();

	const std::multiset<HighScore> *const high_scores
		(HighScores::Instance().Get(window_width, window_height));
	if (!high_scores) {
		ost.str("None.");
		X11::Inst().DrawString(pos, ost.str().c_str());
		return;
	}
	
	int rank (0);
	int last_score (-1);
	for (std::multiset<HighScore>::const_reverse_iterator it (high_scores->rbegin());
	     it != high_scores->rend() && pos.y < window_height;
	     ++it) {
		ost.str("");
		++rank;
		if (it->Value() != last_score) {
			last_score = it->Value();
			ost << std::setfill(' ') << std::setw(2) << rank << ". ";
		} else {
			ost << "    ";
		}
		ost << std::setfill(' ') << std::setw(6) << last_score
		    << "   " << it->Name();
		X11::Inst().DrawString(pos, ost.str().c_str());
		pos.y += LineHeight();
		if (color.red >= 1024) {
			color.red -= 1024;
			X11::Inst().AllocColorAlways(&color);
			X11::Inst().SetForeground(color.pixel);
		}
	}
}
