#include "posix_stream.h"
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <algorithm>
#include <utility>


const int PosixBuf::in_back;


PosixBuf::PosixBuf(const std::string & file_name, int open_flags, int mode)
: fd_(open(file_name.c_str(), open_flags, mode))
{
	setp(outbuffer_, outbuffer_ + sizeof outbuffer_ - 1);
	setg(inbuffer_ + in_back, inbuffer_ + in_back, inbuffer_ + in_back);
}


PosixBuf::~PosixBuf()
{
	if (fd_ >= 0) {
		sync();
		close(fd_);
	}
}


int PosixBuf::Lock(short type) const
{
	int res;
	struct flock lk;
	lk.l_type = type;
	lk.l_whence = SEEK_CUR;
	lk.l_start = 0;
	lk.l_len = 0;
	while ((res = fcntl(fd_, F_SETLKW, &lk)) == -1 && errno == EINTR);
	return res;
}


int PosixBuf::Truncate(off_t len) const
{
	return ftruncate(fd_, len);
}


PosixBuf::int_type PosixBuf::FlushBuffer()
{
	const int total (pptr() - pbase());
	int flushed (0);
	do {
		ssize_t ret;
		while ((ret = write(fd_, outbuffer_ + flushed, total - flushed)) == -1
		       && errno == EINTR);
		if (ret == -1) return traits_type::eof();
		flushed += ret;
	} while (flushed < total);
	pbump(-total);
	return total;
}


PosixBuf::int_type PosixBuf::overflow(int_type c)
{
	if (c != traits_type::eof()) {
		*pptr() = c;
		pbump(1);
	}
	if (FlushBuffer() == traits_type::eof()) {
		return traits_type::eof();
	}
	return c;
}


PosixBuf::int_type PosixBuf::underflow()
{
	if (gptr() < egptr()) return static_cast<unsigned char>(*gptr());
	const int n_back (std::min(in_back, gptr() - eback()));
	std::copy(gptr() - n_back, gptr(), inbuffer_ + 4 - n_back);
	int ret;
	while ((ret = read(fd_, inbuffer_ + in_back, sizeof inbuffer_ - in_back)) == -1
	       && errno == EINTR);
	if (ret <= 0) return traits_type::eof();
	setg(inbuffer_ + in_back - n_back,
	     inbuffer_ + in_back,
	     inbuffer_ + in_back + ret);
	return static_cast<unsigned char>(*gptr());
}


int PosixBuf::sync()
{
	if (FlushBuffer() == traits_type::eof()) return -1;
	return 0;
}


PosixBuf::pos_type PosixBuf::seekpos(pos_type pos, std::ios_base::openmode mode)
{
//	if (mode != std::ios_base::in | std::ios_base::out) return -1;
	if (FlushBuffer() == traits_type::eof()) return -1;
	return lseek(fd_, pos, SEEK_SET);
}
